package org.openjdk.gcbench.runtime.reads;

import org.openjdk.jmh.annotations.*;

import java.util.concurrent.TimeUnit;

import static org.openjdk.gcbench.util.Sinks.sink;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Threads(1)
@State(Scope.Benchmark)
public class Volatile {

    Target target;

    @Setup
    public void setup() {
        target = new Target();
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_none() {
        sink();
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_boolean() {
        sink(target.f_boolean);
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_byte() {
        sink(target.f_byte);
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_char() {
        sink(target.f_char);
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_short() {
        sink(target.f_short);
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_int() {
        sink(target.f_int);
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_float() {
        sink(target.f_float);
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_long() {
        sink(target.f_long);
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_double() {
        sink(target.f_double);
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_Object() {
        sink(target.f_Object);
    }

    static class Target {
        volatile boolean f_boolean;
        volatile byte f_byte;
        volatile short f_short;
        volatile char f_char;
        volatile int f_int;
        volatile float f_float;
        volatile long f_long;
        volatile double f_double;
        volatile Object f_Object;
    }

}
